@@ -1,9 +1,9 @@
 pinvsm <- 
-function(X,tol=.Machine$double.eps){
+function(X,iconst=-1,tol=.Machine$double.eps){
   # pseudoinverse for symmetric matrices
   
   xeig <- eigen(X,symmetric=TRUE)
   nze <- sum(xeig$val>xeig$val[1]*tol)
-  Xinv <- xeig$vec[,1:nze]%*%((xeig$val[1:nze]^-1)*t(xeig$vec[,1:nze]))
+  Xinv <- xeig$vec[,1:nze]%*%((xeig$val[1:nze]^iconst)*t(xeig$vec[,1:nze]))
   
 }
\ No newline at end of file
